//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import gwe.sql.gweMysql;
import gwe.sql.gweMysqlColumn;
import gwe.sql.gweMysqlException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.io.Reader;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
//import java.util.Map;
import java.util.Calendar;
import java.util.Hashtable;

/**
 * This class implements the JDBC draft specification for the ResulSet
 * interface for mysql.  The class is designed to hold the results of
 * database queries.
 */
public class gweMysqlResultSetCore implements java.sql.ResultSet {
  public static final int TYPE_FORWARD_ONLY = 1003;
  public static final int TYPE_SCROLL_INSENSITIVE = 1004;
  public static final int TYPE_SCROLL_SENSITIVE = 1005;
  public static final int CONCUR_READ_ONLY = 1007;
  public static final int CONCUR_UPDATABLE = 1008;
  public static final int FETCH_FORWARD = 1000;
  public static final int FETCH_REVERSE = 1001;
  public static final int FETCH_UNKNOWN = 1002;

  private gweMysql result;
  /**
   * The meta data for this result set.
   */
  private gweMysqlMetaResult meta;
  private Hashtable columnName;
  private Hashtable columnFullName;
  /**
   * Keeps track of the last column read.
   */
  private int last_read = -1;
  /**
   * result set type
   */
  private int resultSetType;
  /**
   * result set concurrency
   */
  private int resultSetConcurrency;
  /**
   * a hint as to the direction in which the rows in a result set
   */
  private int fetchDirection;
  /**
   * a hint as to the number of rows that should
   * be fetched from the database when more rows are needed.
   */
  private int fetchSize;
  /**
   * the Statement that produced this ResultSet object
   */
  private Statement statement;

  private boolean packet_done = false;

  /**
   * Constructs a new result set object given the gweMysqlResult specified.
   * @param res the gweMysql returned from a previously executed query
   * @param rs_type the result set type
   * @param rs_concur the result set concurrency
   */
  public gweMysqlResultSetCore(gweMysql res, Statement st, int rs_type, int rs_concur) {
    result = res;
    statement = st;
    resultSetType = rs_type;
    resultSetConcurrency = rs_concur;
    fetchDirection = gweMysqlResultSetCore.FETCH_UNKNOWN;
    fetchSize = 0;

    meta = new gweMysqlMetaResult(result);
    columnName = (Hashtable)result.columnName.clone();
    columnFullName = (Hashtable)result.columnFullName.clone();
  }

  /**
   * JDBC draft specification method for moving the current row to the
   * next row, returning true if there is another row for processing.
   * @see java.sql.ResultSet#next
   * @exception SQLException thrown if an error occurs during processing
   * @return true if there are more rows to process, otherwise false
   */
  public synchronized boolean next() throws SQLException {
    if( packet_done == true )
      return false;
    try {
      last_read=-1;
      boolean retval = result.nextResult();
      if( retval == false )
        packet_done = true;
      return retval;
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC specification method to determine if a column is null.
   * @see java.sql.ResultSet#isNull
   * @exception SQLException in the event of an gweMysqlException
   * @param column the column being tested for nullness
   * @return true if the column is null, false otherwise
   */
  public boolean isNull(int column) throws SQLException {
    try {
    return (result.isNull(column));
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC specification method to determine if the last column read was null.
   * @see java.sql.ResultSet#wasNull
   * @exception SQLException in the event of an gweMysqlException
   * @return true if the column was null, false otherwise
   */
  public boolean wasNull() throws SQLException {
    if( last_read == -1 ) return false;
    return isNull(last_read);
  }

  /**
   * JDBC draft specification method for getting a string value from
   * the named column.  Note that the JDBC draft provides that this
   * method gets the value as a char, so you can retrieve int values
   * into String objects.
   * @see java.sql.ResultSet#getString
   * @exception SQLException thrown for invalid columns or bad rows
   * @param column the column being retrieved
   * @return the column as a String
   */
  public String getString(int column) throws SQLException {
    last_read = column;
    if( isNull(column) ) return null;
    try {
//    return result.getString(column);

      try{
        byte[] b = result.getBytes(column);
        int b_len = b.length;
        String enc = result.getEncoding();
        gweNativeUnicodeMapper mapperclass = result.getNativeUnicodeMapper();
        String retval = "";
        while( b_len > 0 )
        {
          if( mapperclass != null )
            retval = new String(mapperclass.Native2Unicode(b));
          else
            retval = new String(b, 0, b_len, enc);
          if( retval.length() > 0 )
            break;
          b_len--;
        }
        return retval;
      }
      catch(UnsupportedEncodingException enc_e){
        ;
      }
      return result.getString(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * Get the value of a column in the current row as a Java byte.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @return the column value; if the value is SQL NULL the result is 0
   */
  public byte getByte(int column) throws SQLException {
    String str;

    last_read = column;
    if( isNull(column) ) return 0;
    try {
      str = result.getString(column);
      if( str.equals("") ) return 0;
      return (byte)(Integer.valueOf(str).intValue());
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC specification method for retrieving a column as a boolean
   * value.  Interprets "", null, and "0" as false, others as true.
   * @see java.sql.ResultSet#getBoolean
   * @exception SQLException a sure sign of the apocolypse
   * @param column the column for which the value is being retrieved
   * @return false for "", null, or "0"; true otherwise
   */
  public boolean getBoolean(int column) throws SQLException {
    last_read = column;
    if( isNull(column) ) return false;
    try {
      String str = result.getString(column);

      if( str.equals("") ) return false;
      if( str.equals("0") ) return false;
      if( str.equals("N") ) return false;
      if( str.equals("n") ) return false;
      return true;
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve a short value from
   * the database.
   * @see java.sql.ResultSet#getShort
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public short getShort(int column) throws SQLException {
    try {
    String str;
    last_read=column;
    str = result.getString(column);
    if( str == null ) return 0;
      return (short)(Integer.valueOf(str).intValue());
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve an integer value from
   * the database.
   * @see java.sql.ResultSet#getInt
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as an integer
   */
  public int getInt(int column) throws SQLException {
    try {
    String str;
    last_read=column;
    str = result.getString(column);
    if( str == null ) return 0;
        return Integer.valueOf(str).intValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve a long value from
   * the database.
   * @see java.sql.ResultSet#getLong
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a short
   */
  public long getLong(int column) throws SQLException {
    try {
    String str;
    last_read=column;
    str = result.getString(column);
    if( str == null ) return 0;
      return Long.valueOf(str).longValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve a float value from
   * the database.
   * @see java.sql.ResultSet#getFloat
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a float
   */
  public float getFloat(int column) throws SQLException {
    try {
    String str;
    last_read=column;
    str = result.getString(column);
    if( str == null ) return 0;
        return Float.valueOf(str).floatValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method to retrieve a double value from
   * the database.
   * @see java.sql.ResultSet#getDouble
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @return the named column as a double
   */
  public double getDouble(int column) throws SQLException {
    try {
      String str;
      last_read=column;
      str = result.getString(column);
      if( str == null ) return 0;
        return Double.valueOf(str).doubleValue();
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }


  /* @deprecated */
  public BigDecimal getBigDecimal(int column, int scale) throws SQLException {
    try {
    String str;
    last_read=column;
    str = result.getString(column);
    if( str == null ) return new BigDecimal (new BigInteger ("0"), scale);
    else return new BigDecimal (new BigInteger (str), scale);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }


  /* @deprecated */
  public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
    return getBigDecimal(findColumn(columnName), scale);
  }


  /**
   * JDBC draft specification method to retrieve a Bignum object from
   * the database.
   * @see java.sql.ResultSet#getBignum
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getBignum(int column, int scale) throws SQLException {
    try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return new Bignum(0, scale);
    else return new Bignum(str, scale);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to retrieve a decimal from
   * the database.
   * @see java.sql.ResultSet#getDecimal
   * @exception SQLException things did not go so hot
   * @param column the column being retrieved
   * @param scale how many decimal digits after the floating point to maintain
   * @return the named column as a Bignum
  public Bignum getDecimal(int column, int scale) throws SQLException {
    try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return new Bignum(0, scale);
    else return new Bignum(str, scale);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }
   */


  /**
   * JDBC draft specification method to return a byte array.
   * @see java.sql.ResultSet#getBytes
   * @exception SQLException thrown if something goes wrong
   * @param column the column being retrieved
   * @return a byte array that is the value of the column
   */

  public byte[] getBytes(int column) throws SQLException {
    try {
      last_read = column;
      return result.getBytes(column);
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC 1.10 specification for retrieving a date column.
   * Can you say namespace pollution?  I knew you could.
   * @see java.sqlResultSet#getDate
   * @exception SQLException thrown in the event of problems
   * @param column the column being retrieved
   * @return the date value for the column
   */
  public java.sql.Date getDate(int column)
       throws SQLException {
  try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return null;
    /*
    Integer y = new Integer(str.substring(0,2));
    Integer m = new Integer(str.substring(2,4));
    Integer d = new Integer(str.substring(4,6));
    return new java.sql.Date(y.intValue(), m.intValue()-1, d.intValue());
    */
    return java.sql.Date.valueOf(str);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for retrieving a time from the database.
   * @see java.sql.ResultSet#getTime
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a java.sql.Time object
   */
  public java.sql.Time getTime(int column)
       throws SQLException {
  try {
    String str;
    last_read = column;
    str = result.getString(column);
    if( str == null ) return null;
    /*
    Integer h = new Integer(str.substring(6,8));
    Integer m = new Integer(str.substring(8,10));
    Integer s = new Integer(str.substring(10,12));
    return new java.sql.Time(h.intValue(), m.intValue(), s.intValue());
    */
    return java.sql.Time.valueOf(str);
  }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for retrieving a timestamp from
   * the database.
   * @see java.sql.ResultSet#getTimestamp
   * @exception SQLException thrown in the event of troubles
   * @param column the column being retrieved
   * @return the column as a java.sql.Timestamp object
   */
  public java.sql.Timestamp getTimestamp(int column) throws SQLException {
    try {
      last_read = column;
      String str = result.getString(column);
      if( str == null )
        return null;

      try {
        return java.sql.Timestamp.valueOf(str);
      }
      catch(Exception e) {
        // Thanks for INDEN-san
        if( str.length() == 14 )
        {
          Integer y  = new Integer(str.substring(0,4));
          Integer m  = new Integer(str.substring(4,6));
          Integer d  = new Integer(str.substring(6,8));
          Integer hr = new Integer(str.substring(8,10));
          Integer mi = new Integer(str.substring(10,12));
          Integer se = new Integer(str.substring(12,14));
          return new java.sql.Timestamp(y.intValue()-1900, m.intValue()-1, d.intValue(), hr.intValue(), mi.intValue(), se.intValue(), 0);
        }
        else if( str.length() == 12 )
        {
          Integer y  = new Integer(str.substring(0,2));
          Integer m  = new Integer(str.substring(2,4));
          Integer d  = new Integer(str.substring(4,6));
          Integer hr = new Integer(str.substring(6,8));
          Integer mi = new Integer(str.substring(8,10));
          Integer se = new Integer(str.substring(10,12));
          int two_digit_y = y.intValue();
          if( two_digit_y >= 0 && two_digit_y <= 69 )
            two_digit_y += 100;
          return new java.sql.Timestamp(two_digit_y, m.intValue()-1, d.intValue(), hr.intValue(), mi.intValue(), se.intValue(), 0);
        }
        else
        {
          throw new SQLException(e.getMessage());
        }
      }
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getAsciiStream(int column)
       throws SQLException {
    throw new SQLException("mysql does not support getAsciiStream(int).");
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getUnicodeStream(int column)
       throws SQLException {
    throw new SQLException("mysql does not support getUnicodeStream(int).");
  }

  /**
   * This is not currently supported.
   */
  public java.io.InputStream getBinaryStream(int column)
       throws SQLException {
    try{
      byte[] bin_buf = getBytes(column);
      if( bin_buf == null )
        return null;
      return new java.io.ByteArrayInputStream(bin_buf);
    }
    catch(Exception e) {
      throw new SQLException(e.getMessage());
    }
//    return null;
  }

  /**
   * JDBC draft specification method for closing a result set.
   * @see java.sql.ResultSet#close
   */
  public void close() throws SQLException {
    // We need to halt the flow of result packets -
    // for now, close the socket, and re-conect.
    try {
//    result.reConnect();
      if( packet_done == false )
      {
        while(true)
        {
          if( this.next() == false )
            break;
        }
      }
    }
    catch( Exception e ) {
    throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification method for returning a cursor name.
   * @see java.sql.ResultSet#getCursorName
   * @return ""
   */
  public String getCursorName() throws SQLException {
    return "";
  }

  /**
   * JDBC draft specification method for returning meta-deta on a result
   * set.
   * @see java.sql.ResultSet#getMetaData
   * @exception SQLException thrown on error getting meta-data
   * @return ResultSetMetaData object containing result set info
   */
  public ResultSetMetaData getMetaData() throws SQLException {
//    if( meta == null ) {
//      meta = new gweMysqlMetaResult(result);
//    }
    return meta;
  }

  /**
   * JDBC draft specification method for retrieving data as objects.
   * @see java.sql.ResultSet#getObject
   * @exception SQLException in the event of retrieval! error
   * @param column the column desired
   * @param type the SQL data type of the field
   * @scale preceision for Bignums
   * @return the column specified as an Object
   */
  public Object getObject(int column, int type, int scale)
       throws SQLException {
    switch(type) {
    case Types.BIT:
      return new Boolean(getBoolean(column));

    case Types.TINYINT:
      return new Byte((byte)getInt(column));

    case Types.SMALLINT:
      return new Integer(getShort(column));

    case Types.INTEGER:
      return new Integer(getInt(column));

    case Types.BIGINT:
      return new Long(getLong(column));

    case Types.FLOAT:
      return new Float(getFloat(column));

    case Types.REAL:
      return new Float(getFloat(column));

    case Types.DOUBLE:
      return new Double(getDouble(column));

    case Types.NUMERIC:
      return getBigDecimal(column, scale);

    case Types.DECIMAL:
      return getBigDecimal(column, scale);

    case Types.CHAR:
      return getString(column);

    case Types.VARCHAR:
      return getString(column);

    case Types.LONGVARCHAR:
      return getString(column);

    case Types.DATE:
      return getDate(column);

    case Types.TIME:
      return getTime(column);

    case Types.TIMESTAMP:
      return getTimestamp(column);

    case Types.BINARY:
      return getBytes(column);

    case Types.VARBINARY:
      return getBytes(column);

    case Types.LONGVARBINARY:
      return getBytes(column);

    default:
      return null;
    }
  }

  /**
   * Same as above, except with a default scale to 10.
   */
  public Object getObject(int column, int type)
       throws SQLException {
    return getObject(column, type, 10);
  }

  /**
   * Same as above, except using the column's default SQL type.
   */
  public Object getObject(int column) throws SQLException {
    ResultSetMetaData meta = getMetaData();
    int type = meta.getColumnType(column);

    return getObject(column, type);
  }

  /**
   * JDBC 0.65 specification method for retrieving a column value
   * as a char based on the column name.
   * @see java.sql.ResultSet#getString
   * @param name the name of the column desired
   * @return the value of the column as a char
   */
  public String getString(String name) throws SQLException {
    return getString(findColumn(name));
  }

  /**
   * Returns the column as a byte based on column name
   */
  public byte getByte(String columnName) throws SQLException {
    return getByte(findColumn(columnName));
  }

  /**
   * Get the value of a BIT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is false
   */
  public boolean getBoolean(String columnName) throws SQLException {
    return getBoolean(findColumn(columnName));
  }

  /**
   * Get the value of a short by column name
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public short getShort(String columnName) throws SQLException {
    return getShort(findColumn(columnName));
  }

  /**
   * Get the value of a INTEGER column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public int getInt(String columnName) throws SQLException {
    return getInt(findColumn(columnName));
  }

  /**
   * Get the value of a long column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public long getLong(String columnName) throws SQLException {
    return getLong(findColumn(columnName));
  }

  /**
   * Get the value of a FLOAT column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public float getFloat(String columnName) throws SQLException {
    return getFloat(findColumn(columnName));
  }

  /**
   * Get the value of a DOUBLE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is 0
   */
  public double getDouble(String columnName) throws SQLException {
    return getDouble(findColumn(columnName));
  }


  /**
   * Get the value of a NUMERIC column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null

  public Bignum getBignum(String columnName, int scale) throws SQLException {
    return getBignum(findColumn(columnName), scale);
  }
   */


  /**
   * Get the value of a BINARY column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public byte[] getBytes(String columnName) throws SQLException {
    return getBytes(findColumn(columnName));
  }

  /**
   * Get the value of a DATE column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public java.sql.Date getDate(String columnName) throws SQLException {
    return getDate(findColumn(columnName));
  }

  /**
   * Get the value of a TIME column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public java.sql.Time getTime(String columnName) throws SQLException {
    return getTime(findColumn(columnName));
  }

  /**
   * Get the value of a TIMESTAMP column in the current row
   *
   * @param columnName is the SQL name of the column
   * @return the column value; if isNull the value is null
   */
  public java.sql.Timestamp getTimestamp(String columnName)
       throws SQLException {
    return getTimestamp(findColumn(columnName));
  }

  /**
   * Very large ascii values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the ascii value
   */
  public java.io.InputStream getAsciiStream(String columnName)
       throws SQLException {
    return getAsciiStream(findColumn(columnName));
  }

  /**
   * Very large unicode values in a LONGVARCHAR column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the unicode value
   */
  public java.io.InputStream getUnicodeStream(String columnName)
       throws SQLException {
    return getUnicodeStream(findColumn(columnName));
  }

  /**
   * Very large binary values in a LONGVARBINARY column can
   * be read in small chunks from a java.io.InputStream.
   *
   * <P><B>Note:</B> All the data in the returned stream must
   * be read prior to getting the value of any other column. The
   * next call to a get method implicitly closes the stream.
   *
   * @param columnName is the SQL name of the column
   * @return the java input stream which contains the binary value */
  public java.io.InputStream getBinaryStream(String columnName)
       throws SQLException {
    return getBinaryStream(findColumn(columnName));
  }

  /**
   * Get the value of a column as an object; an integral value is
   * returned as its java.lang equivalent object; a LONGVARCHAR or
   * LONGVARBINARY value is returned as a java.io.InputStream.
   *
   * @param columnName the SQL column name
   * @param sqlType SQL type code defined by java.sql.Types
   * @return the parameter value; if isNull the value is null
   */
  public Object getObject(String columnName, int sqlType, int scale)
       throws SQLException {
    return getObject(findColumn(columnName), sqlType, scale);
  }

  /**
   * Same as above, except defaulting scale to 0.
   */
  public Object getObject(String columnName, int type)
       throws SQLException {
    return getObject(findColumn(columnName), type, 0);
  }

  /**
   * Same as above, except returning the default SQL type
   */
  public Object getObject(String columnName) throws SQLException {
    return getObject(findColumn(columnName));
  }

  /**
   * column name counter part for other isNull()
   */
  public boolean isNull(String name) throws SQLException {
    return isNull(findColumn(name));
  }

  /**
   * Given a column name, this method returns the column number for that
   * name.  Column name to number mappings are kept inside a Hashtable.
   * Applications that do not need the overhead of this calculation are
   * not penalized since the mapping only occurs on the first attempt to
   * access a column number by name.
   * @exception java.sql.SQLException thrown if a bad name is passed
   * @param name the name of the column desired
   * @return the column number, 1 being the first column
   */
  public int findColumn(String name) throws SQLException {
    try {
//      return result.getColumnNum(name);
      Integer nI = (Integer) columnName.get(name);
      if (nI==null) nI = (Integer) columnFullName.get(name);
      if (nI!=null) return nI.intValue()+1;
      throw new SQLException("Invalid column name : "+name);
    }
    catch( Exception e ) {
      throw new SQLException(e.getMessage());
    }
  }

  /**
   * JDBC draft specification for getting the chain of warnings for this
   * statement.
   * @see java.sql.Statement#getWarnings
   * @return the chain of warnings
   */
  public SQLWarning getWarnings() throws SQLException {
    return null;
  }

  /**
   * JDBC draft specification for clearing the warning chain.
   * @see java.sql.Statement#clearWarnings
   */
  public void clearWarnings() throws SQLException {
  }

  String getEncoding() {
    return result.getEncoding();
  }

  gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return result.getNativeUnicodeMapper();
  }

  //--------------------------JDBC 2.0-----------------------------------

  public Reader getCharacterStream(int columnIndex) throws SQLException {
    return null;
  }

  public Reader getCharacterStream(String columnName) throws SQLException {
    return getCharacterStream(findColumn(columnName));
  }

  public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
    return getBigDecimal(columnIndex, 10);
  }

  public BigDecimal getBigDecimal(String columnName) throws SQLException {
    return getBigDecimal(findColumn(columnName));
  }

  public boolean isBeforeFirst() throws SQLException {
    throw new SQLException("mysql does not support isBeforeFirst().");
  }

  public boolean isAfterLast() throws SQLException {
    throw new SQLException("mysql does not support isAfterLast().");
  }

  public boolean isFirst() throws SQLException {
    throw new SQLException("mysql does not support isFirst().");
  }

  public boolean isLast() throws SQLException {
    throw new SQLException("mysql does not support isLast().");
  }

  public void beforeFirst() throws SQLException {
    throw new SQLException("mysql does not support beforeFirst().");
  }

  public void afterLast() throws SQLException {
    throw new SQLException("mysql does not support afterLast().");
  }

  public boolean first() throws SQLException {
    throw new SQLException("mysql does not support first().");
  }

  public boolean last() throws SQLException {
    throw new SQLException("mysql does not support last().");
  }

  public int getRow() throws SQLException {
    return 0;
  }

  public boolean absolute(int row) throws SQLException {
    throw new SQLException("mysql does not support absolute(int).");
  }

  public boolean relative(int rows) throws SQLException {
    throw new SQLException("mysql does not support relative(int).");
  }

  public boolean previous() throws SQLException {
    throw new SQLException("mysql does not support previous().");
  }

  public void setFetchDirection(int direction) throws SQLException {
    fetchDirection = direction;
  }

  public int getFetchDirection() throws SQLException {
    return fetchDirection;
  }

  public void setFetchSize(int rows) throws SQLException {
    fetchSize = rows;
  }

  public int getFetchSize() throws SQLException {
    return fetchSize;
  }

  public int getType() throws SQLException {
    return resultSetType;
  }

  public int getConcurrency() throws SQLException {
    return resultSetConcurrency;
  }

  public boolean rowUpdated() throws SQLException {
    throw new SQLException("mysql does not support rowUpdated().");
  }

  public boolean rowInserted() throws SQLException {
    throw new SQLException("mysql does not support rowInserted().");
  }

  public boolean rowDeleted() throws SQLException {
    throw new SQLException("mysql does not support rowDeleted().");
  }

  public void updateNull(int columnIndex) throws SQLException {
    throw new SQLException("mysql does not support updateNull(int).");
  }

  public void updateBoolean(int columnIndex, boolean x) throws SQLException {
    throw new SQLException("mysql does not support updateBoolean(int, boolean).");
  }

  public void updateByte(int columnIndex, byte x) throws SQLException {
    throw new SQLException("mysql does not support updateByte(int, byte).");
  }

  public void updateShort(int columnIndex, short x) throws SQLException {
    throw new SQLException("mysql does not support updateShort(int, short).");
  }

  public void updateInt(int columnIndex, int x) throws SQLException {
    throw new SQLException("mysql does not support updateInt(int, int).");
  }

  public void updateLong(int columnIndex, long x) throws SQLException {
    throw new SQLException("mysql does not support updateLong(int, long).");
  }

  public void updateFloat(int columnIndex, float x) throws SQLException {
    throw new SQLException("mysql does not support updateFloat(int, float).");
  }

  public void updateDouble(int columnIndex, double x) throws SQLException {
    throw new SQLException("mysql does not support updateDouble(int, double).");
  }

  public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {
    throw new SQLException("mysql does not support updateBigDecimal(int, BigDecimal).");
  }

  public void updateString(int columnIndex, String x) throws SQLException {
    throw new SQLException("mysql does not support updateString(int, String).");
  }

  public void updateBytes(int columnIndex, byte[] x) throws SQLException {
    throw new SQLException("mysql does not support updateBytes(int, byte[]).");
  }

  public void updateDate(int columnIndex, Date x) throws SQLException {
    throw new SQLException("mysql does not support updateDate(int, Date).");
  }

  public void updateTime(int columnIndex, Time x) throws SQLException {
    throw new SQLException("mysql does not support updateTime(int, Time).");
  }

  public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {
    throw new SQLException("mysql does not support updateTimestamp(int, Timestamp).");
  }

  public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateAsciiStream(int, InputStream, int).");
  }

  public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateBinaryStream(int, InputStream, int).");
  }

  public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateCharacterStream(int, Reader, int).");
  }

  public void updateObject(int columnIndex, Object x, int scale) throws SQLException {
    throw new SQLException("mysql does not support updateObject(int, Object, int).");
  }

  public void updateObject(int columnIndex, Object x) throws SQLException {
    throw new SQLException("mysql does not support updateObject(int, Object).");
  }

  public void updateNull(String columnName) throws SQLException {
    throw new SQLException("mysql does not support updateNull(String).");
  }

  public void updateBoolean(String columnName, boolean x) throws SQLException {
    throw new SQLException("mysql does not support updateBoolean(String, boolean).");
  }

  public void updateByte(String columnName, byte x) throws SQLException {
    throw new SQLException("mysql does not support updateByte(String, byte).");
  }

  public void updateShort(String columnName, short x) throws SQLException {
    throw new SQLException("mysql does not support updateShort(String, short).");
  }

  public void updateInt(String columnName, int x) throws SQLException {
    throw new SQLException("mysql does not support updateInt(String, int).");
  }

  public void updateLong(String columnName, long x) throws SQLException {
    throw new SQLException("mysql does not support updateLong(String, long).");
  }

  public void updateFloat(String columnName, float x) throws SQLException {
    throw new SQLException("mysql does not support updateFloat(String, float).");
  }

  public void updateDouble(String columnName, double x) throws SQLException {
    throw new SQLException("mysql does not support updateDouble(String, double).");
  }

  public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {
    throw new SQLException("mysql does not support updateBigDecimal(String, BigDecimal).");
  }

  public void updateString(String columnName, String x) throws SQLException {
    throw new SQLException("mysql does not support updateString(String, String).");
  }

  public void updateBytes(String columnName, byte[] x) throws SQLException {
    throw new SQLException("mysql does not support updateBytes(String, byte[]).");
  }

  public void updateDate(String columnName, Date x) throws SQLException {
    throw new SQLException("mysql does not support updateDate(String, Date).");
  }

  public void updateTime(String columnName, Time x) throws SQLException {
    throw new SQLException("mysql does not support updateTime(String, Time).");
  }

  public void updateTimestamp(String columnName, Timestamp x) throws SQLException {
    throw new SQLException("mysql does not support updateTimestamp(String, Timestamp).");
  }

  public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateAsciiStream(String, InputStream, int).");
  }

  public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("mysql does not support updateBinaryStream(String, InputStream, int).");
  }

  public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException {
    throw new SQLException("mysql does not support updateCharacterStream(String, Reader, int).");
  }

  public void updateObject(String columnName, Object x, int scale) throws SQLException {
    throw new SQLException("mysql does not support updateObject(String, Object, int).");
  }

  public void updateObject(String columnName, Object x) throws SQLException {
    throw new SQLException("mysql does not support updateObject(String, Object).");
  }

  public void insertRow() throws SQLException {
    throw new SQLException("mysql does not support insertRow().");
  }

  public void updateRow() throws SQLException {
    throw new SQLException("mysql does not support updateRow().");
  }

  public void deleteRow() throws SQLException {
    throw new SQLException("mysql does not support deleteRow().");
  }

  public void refreshRow() throws SQLException {
    throw new SQLException("mysql does not support refreshRow().");
  }

  public void cancelRowUpdates() throws SQLException {
    throw new SQLException("mysql does not support cancelRowUpdates().");
  }

  public void moveToInsertRow() throws SQLException {
    throw new SQLException("mysql does not support moveToInsertRow().");
  }

  public void moveToCurrentRow() throws SQLException {
    throw new SQLException("mysql does not support moveToCurrentRow().");
  }

  public Statement getStatement() throws SQLException {
    return statement;
  }

/*
  public Object getObject(int i, Map map) throws SQLException {
    throw new SQLException("mysql does not support getObject(int, Map).");
  }

  public Ref getRef(int i) throws SQLException {
    throw new SQLException("mysql does not support getRef(int).");
  }

  public Blob getBlob(int i) throws SQLException {
    throw new SQLException("mysql does not support getBlob(int).");
  }

  public Clob getClob(int i) throws SQLException {
    throw new SQLException("mysql does not support getClob(int).");
  }

  public Array getArray(int i) throws SQLException {
    throw new SQLException("mysql does not support getArray(int).");
  }

  public Object getObject(String colName, Map map) throws SQLException {
    throw new SQLException("mysql does not support getObject(String, Map).");
  }

  public Ref getRef(String colName) throws SQLException {
    throw new SQLException("mysql does not support getRef(String).");
  }

  public Blob getBlob(String colName) throws SQLException {
    throw new SQLException("mysql does not support getBlob(String).");
  }

  public Clob getClob(String colName) throws SQLException {
    throw new SQLException("mysql does not support getClob(String).");
  }

  public Array getArray(String colName) throws SQLException {
    throw new SQLException("mysql does not support getArray(String).");
  }
*/

  public Date getDate(int columnIndex, Calendar cal) throws SQLException {
    return getDate(columnIndex);
  }

  public Date getDate(String columnName, Calendar cal) throws SQLException {
    return getDate(findColumn(columnName));
  }

  public Time getTime(int columnIndex, Calendar cal) throws SQLException {
    return getTime(columnIndex);
  }

  public Time getTime(String columnName, Calendar cal) throws SQLException {
    return getTime(findColumn(columnName));
  }

  public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
    return getTimestamp(columnIndex);
  }

  public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException {
    return getTimestamp(findColumn(columnName));
  }
}
