//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import gwe.sql.gweMysql;
import gwe.sql.gweMysqlException;
import java.util.Vector;
import java.util.Enumeration;

/**
 * This class is the implementation of the JDBC draft specification
 * Statement interface.  Statement objects are used for executing SQL
 * statements.
 */
public class gweMysqlStatement implements java.sql.Statement {
  /**
   * A connection object to direct information to.
   */
  private gweMysqlConnection connection;
  /**
   * A result set returned from this query or whatever.
   */
  private gweMysqlResultSetBuffer result;
  /**
   * The max field size for mysql
   * NOTE: This is the value of MAXBUFFER in gweMysql.packet
   */
  private int max_field_size = 65535;
  /**
   * The max rows supported by mysql - big!
   */
  private int max_rows = 50000000;   //Because it's in the FAQ!
  /**
   * The number of seconds the driver will allow for a SQL statement to
   * execute before giving up.  The default is to wait forever (0).
   */
  private int timeout = 0;
  /**
   * a hint as to the direction in which the rows in a result set
   */
  private int fetchDirection;
  /**
   * a hint as to the number of rows that should
   * be fetched from the database when more rows are needed.
   */
  private int fetchSize;
  /**
   * result set type
   */
  private int resultSetType;
  /**
   * result set concurrency
   */
  private int resultSetConcurrency;
  /**
   * batch of commmands
   */
  private java.util.Vector batch = null;

  /**
   * Constructs a new gweMysqlStatement object.
   * @param conn the gweMysqlConnection object
   * @param rs_type the result set type
   * @param rs_concur the result set concurrency
   */
  public gweMysqlStatement(gweMysqlConnection conn,
                           int rs_type, int rs_concur) {
    connection = conn;
    resultSetType = rs_type;
    resultSetConcurrency = rs_concur;
    fetchDirection = gweMysqlResultSetCore.FETCH_UNKNOWN;
    fetchSize = 0;
  }

  /**
   * JDBC draft specification method to execute the specified SQL
   * query and gives back a result set.
   * @see java.sql.Statement#executeQuery
   * @exception SQLException raised for any gweMysqlException
   * @param sql the SQL statement string
   * @return the result set from the query
   */
  public ResultSet executeQuery(String sql) throws SQLException {
    result = null;
    // to avoid creating new resultset before close old resultset!!
    connection.setCurResultSet(null);
    gweMysql r = connection.executeQuery(sql);
    gweMysqlResultSetCore curResultSet =
       new gweMysqlResultSetCore(r, this, resultSetType, resultSetConcurrency);
    connection.setCurResultSet(curResultSet);
    return new gweMysqlResultSetBuffer(curResultSet);
  }

  /**
   * JDBC 0.54 specification method for executing simple UPDATE, INSERT,
   * DELETE, etc. statements which do not return ResultSet's.
   * @see java.sql.Statement#executeUpdate
   * @exception java.sql.SQLException thrown when an error occurs executing
   * the SQL
   * @return 0
   */
  public int executeUpdate(String sql) throws SQLException {
    // to avoid send new SQL statement before close old resultset!!
    connection.setCurResultSet(null);
    return connection.executeUpdate(sql);
  }

  /**
   * JDBC draft specification method to execute an SQL statement and return
   * true if a result set was gotten, false otherwise.
   * @see java.sql.Statement#execute
   * @exception SQLException raised for any gweMysqlException encountered
   * @param sql the SQL to be executed
   * @return true if there is a result set available
   */
  public boolean execute(String sql) throws SQLException {
    // to avoid creating new resultset before close old resultset!!
    connection.setCurResultSet(null);
    gweMysql r = connection.executeQuery(sql);
    if( r == null ) {
      result = null;
    }
    else {
      gweMysqlResultSetCore curResultSet =
       new gweMysqlResultSetCore(r, this, resultSetType, resultSetConcurrency);
      connection.setCurResultSet(curResultSet);
      result = new gweMysqlResultSetBuffer(curResultSet);
    }
    return (result != null);
  }

  /**
   * JDBC draft specification method to close off any open result sets
   * for this Statement.
   * @see java.sql.Statement#close
   */
  public void close() throws SQLException {
  }

  /**
   * JDBC 0.54 specification method for obtaining a result set from a
   * statement object.
   * @see java.sql.Statement#getResultSet
   * @return null if no result set is available, otherwise a result set
   */
  public ResultSet getResultSet() throws SQLException {
    ResultSet r;

    r = result;
    result = null;
    return r;
  }

  /**
   * Returns -1 always for mysql.
   * @see java.sql.Statement#getUpdateCount
   * @return -1
   */
  public int getUpdateCount() throws SQLException {
    return -1;
  }

  /**
   * JDBC 0.54 specification for determining if any more result sets
   * are left from a database statement. 
   * @see java.sql.Statement#getMoreResults
   * @return true if rows are to be gotten
   */
  public boolean getMoreResults() throws SQLException {
    return (result != null);
  }

  /**
   * JDBC draft specification method for determining the number of
   * results sets left to process for multiple result set processing.
   * @see java.sql.Statement#getMoreCounts
   * @see getMoreResults
   * @return -1
   */
  public int getMoreCounts() throws SQLException {
    return -1;
  }

  /**
   * JDBC draft specification method for getting the max field size
   * @see java.sql.Statement#getMaxFieldSize
   * @return the value of max field size
   */
  public int getMaxFieldSize() throws SQLException {
    return max_field_size;
  }

  /**
   * JDBC draft specification method for setting the max field size.
   * @see java.sql.Statement#setMaxFieldSize
   * @param max the maximum field size
   */
  public void setMaxFieldSize(int max) throws SQLException {
    max_field_size = max;
  }

  /**
   * JDBC draft specification method for getting the max rows supported
   * @see java.sql.Statement#getMaxRows
   * @return the maximum rows supported by mysql
   */
  public int getMaxRows() throws SQLException {
    return max_rows;
  }

  /**
   * JDBC draft specification method for setting the max rows.
   * @see java.sql.Statement.setMaxRows
   * @param max the max rows
   */
  public void setMaxRows(int max) throws SQLException {
    max_rows = max;
  }

  /**
   * JDBC draft specification method for escape processing.
   * @see java.sql.Statement#setEscapeProcessing
   * @param enable this does nothing right now
   */
  public void setEscapeProcessing(boolean enable)
       throws SQLException {
    throw new SQLException("mysql does not support escape processing.");
  }

  /**
   * JDBC draft specification method for getting the query timeout, which
   * is the number of seconds the driver waits before giving up on the
   * query.
   * @see java.sql.Statement#getQueryTimeout
   * @see setQueryTimeout
   * @return the timeout value for this statement
   */
  public int getQueryTimeout() throws SQLException {
    return timeout;
  }

  /**
   * JDBC draft specification method for setting the query timeout.
   * @see java.sql.Statement#setQueryTimeout
   * @see getQueryTimeout
   * @param x the new query timeout value
   */
  public void setQueryTimeout(int x) throws SQLException {
    timeout = x;
  }

  /**
   * JDBC draft specification method for allowing one thread to cancel this
   * statement which may be running in another thread.
   * @see java.sql.Statement#cancel
   */
  public void cancel() {
  }

  /**
   * JDBC draft specification for getting the chain of warnings for this
   * statement.
   * @see java.sql.Statement#getWarnings
   * @return the chain of warnings
   */
  public final SQLWarning getWarnings() throws SQLException {
    return null;
  }

  /**
   * JDBC draft specification for clearing the warning chain.
   * @see java.sql.Statement#clearWarnings
   */
  public void clearWarnings() throws SQLException {
  }

  /**
   * JDBC 0.65 specification for setting cursor names.
   * mysql does not support cursors
   */
  public void setCursorName(String unused) throws SQLException {
    throw new SQLException("mysql does not support cursors.");
  }

  //--------------------------JDBC 2.0-----------------------------------

  public void setFetchDirection(int direction) throws SQLException {
    fetchDirection = direction;
  }

  public int getFetchDirection() throws SQLException {
    return fetchDirection;
  }

  public void setFetchSize(int rows) throws SQLException {
    fetchSize = rows;
  }

  public int getFetchSize() throws SQLException {
    return fetchSize;
  }

  public int getResultSetConcurrency() throws SQLException {
    return resultSetConcurrency;
  }

  public int getResultSetType() throws SQLException {
    return resultSetType;
  }

  private void ensureBatch() {
    if( batch == null )
      batch = new java.util.Vector();
  }

  public void addBatch(String sql) throws SQLException {
    ensureBatch();
    batch.addElement(sql);
  }

  public void clearBatch() throws SQLException {
    ensureBatch();
    batch.removeAllElements();
  }

  public int[] executeBatch() throws SQLException {
    ensureBatch();
    int[] retval = new int[batch.size()];

    for(int i = 0; i < retval.length; i++)
    {
      retval[i] = 0;
    }

    int j = 0;
    for(Enumeration e = batch.elements(); e.hasMoreElements(); j++)
    {
      String sql = (String)e.nextElement();
      try{
        retval[j] = executeUpdate(sql);
      }
      catch(SQLException sql_e){
        new BatchUpdateException(retval);
      }
    }

    return (retval);
  }

  public Connection getConnection() throws SQLException {
    return connection;
  }
}

class BatchUpdateException extends SQLException {
  public BatchUpdateException( String reason, String SQLState, int vendorCode, 
			       int[] updateCounts ) {
    super(reason, SQLState, vendorCode);
    this.updateCounts = updateCounts;
  }
  public BatchUpdateException(String reason, String SQLState, 
			      int[] updateCounts) {
    super(reason, SQLState);
    this.updateCounts = updateCounts;
  }
  public  BatchUpdateException(String reason, int[] updateCounts) {
    super(reason);
    this.updateCounts = updateCounts;
  }
  public BatchUpdateException(int[] updateCounts) {
    super();
    this.updateCounts = updateCounts;
  }
  public BatchUpdateException() {
    super();
    this.updateCounts = null;
  }
  public int[] getUpdateCounts() {
    return updateCounts;
  }
  private int[] updateCounts;
}
